package entDownloader;

import java.io.*;
import java.net.UnknownHostException;
import java.text.ParseException;
import java.util.*;

import shell.progressBar.ProgressBar;

public final class Main {
	private static boolean debug = false;
	private static String login;
	private static String password;
	private static final String productName = "ENTDownloader";
	private static final String productVersion = "1.0.0 Beta 3";
	private static final String productAuthor = "Kvin Subileau";
	private static final String productContact = "Kevin.Subileau@etudiant.u-clermont1.fr";
	private ProgressBar pg;
	private ENTDownloader entd;
	
	public Main(){
		if (!debug && System.getProperty ("os.name").contains("Windows")) { //Corrige un bug d'accents sous Windows
			PrintStream psout = null;
			PrintStream pserr = null;
			try {
				psout = new PrintStream(System.out,true,"IBM850");
				pserr = new PrintStream(System.err,true,"IBM850");
			} catch (UnsupportedEncodingException e1) {
			}
			// on commence par changer l'encodage en IBM850 (connu aussi sous l'alias Cp850)
			if (psout != null)
				System.setOut(psout);
			if (pserr != null)
				System.setErr(pserr);
		}

		entd = ENTDownloader.getInstance();
		pg = new ProgressBar(false);
		pg.setVisible(false);
		entd.addObserver(new Observer() {
			boolean enabledWrite = true;
			FS_File file = null;
			@Override
			public void update(Observable o, Object arg) {
				boolean pgvisible;
				String message = null;
				
				switch (((ENTDownloader) o).getStatus()) {
					case START_DOWNLOAD:
						file = (FS_File) arg;
						message = "Tlchargement du fichier " + file.getName() + " en cours...";
						pg.setDeterminate(true);
						break;
					case END_DOWNLOAD:
						enabledWrite = true;
						file = null;
						message = "Tlchargement termin.";
						pg.setVisible(false);
						pg.setDeterminate(false);
						pg.setVisible(false);
						break;
					case CHANGEDIR:
						pg.setVisible(true);
						String path = arg.toString();
						if (path.equals(".."))
							path = "parent";
						else if (path.equals("/") || path.equals("~") || path.isEmpty())
							path = "racine";
						
						if (path.equals("."))
							message = "Actualisation en cours...";
						else
							message = "Chargement du dossier "+path+" en cours...";
						break;
					case CHANGEDIR_END:
						pg.setVisible(false);
						message = "Chargement russi.";
						break;
					case INITIALIZE:
						message = "Authentification russie, initialisation...";
						break;
					case INITIALIZING_END:
						pg.setVisible(false);
						message = "Initialisation termine.";
						break;
					case LOGIN:
						message = "Connexion en cours...";
						break;
					case INVALID_CREDENTIALS:
						pg.setVisible(false);
						break;
					default:
						message = (arg!=null?arg.toString():"");
						break;
				}
				
				if(enabledWrite && message != null && !message.isEmpty()) {
					pgvisible = pg.isVisible();
					if(pgvisible)
						pg.setVisible(false);
					System.out.flush();
					System.out.println("  => " + message);
					if(pgvisible)
						pg.setVisible(pgvisible);
				}
				
				switch (((ENTDownloader) o).getStatus()) {
					case START_DOWNLOAD:
						enabledWrite = false;
						break;
					case DOWNLOADING:
						pg.setValue((int)(((Long)arg) *100/file.getSize()));
						break;
					case LOGIN:
						pg.setVisible(true);
						break;
				}
			}
		});
		
		Scanner sc = new Scanner(System.in);
		System.out.print(productName + " v" + productVersion + "\nVeuillez saisir votre login ENT : ");
		login = sc.nextLine();
		if (debug) {
			System.out.print("Mot de passe : ");
			password = sc.nextLine();
		}
		else {
			try {
				password = PwdConsole.getPasswd("Mot de passe :  ");
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		
		try {
			if(!entd.login(login, password)) {
				System.err.println("Echec d'authentification : Login ou mot de passe incorrect.");
				return;
			}
			cd("/");
		} catch (java.net.UnknownHostException e2) {
			if (pg.isVisible())
				pg.setVisible(false);
			System.err.println("ENTDownloader: Service indisponible: Votre connexion Internet semble rencontrer un problme. Assurez-vous que votre ordinateur est connect  Internet.");
			return;
		}
		
		//Lecture des commandes
		System.out.print("\n" + login + "@ent.u-clermont1.fr:"+entd.getDirectoryPath()+">");
		while (sc.hasNextLine()) {
			String[] command = null;
			try {
				command = splitCommand(sc.nextLine());
			} catch (ParseException e3) {
				System.err.println("ENTDownloader: Commande incomplte.");
			}

			try {
				if (command != null && command.length != 0){			
					if(command[0].equals("ls"))
						ls(entd.getDirectoryContent());
					else if (command[0].equals("cd"))
						cd((command.length > 1)?command[1]:"");
					else if (command[0].equals("get"))
						get((command.length > 1)?command[1]:"", (command.length > 2)?command[2]:null);
					else if (command[0].equals("getall")) { //TODO Creer un dossier eponyme au dossier courant comme destination si cette derniere n'est pas indique au lieu d'enregistrer directement dans le dossier courant local ??
						if(command.length > 1 && (command[1].startsWith("-R") || command[1].startsWith("-r")))
							try {
								System.out.println(entd.getAllFiles((command.length > 2)?command[command.length-1]:null, (command[1].length() > 2)?Integer.parseInt(command[1].substring(2)):-1)+" fichier(s) tlcharg(s)");
							} 
							catch (NumberFormatException e2) {
								System.err.println("ENTDownloader: getall: Un nombre entier est attendu aprs l'option -R.");
							}
							catch (ENTInvalidFS_ElementTypeException e) {
								System.err.println("ENTDownloader: getall: Impossible de crer le rpertoire requis : un fichier portant le mme nom existe.");
							}
							catch (IOException e1) {
								// TODO Auto-generated catch block
								e1.printStackTrace();
							}
						else
							try {
								System.out.println(entd.getAllFiles((command.length > 1)?command[1]:null, 0)+" fichier(s) tlcharg(s)");
							} catch (ENTInvalidFS_ElementTypeException e) {
								System.err.println("ENTDownloader: getall: Impossible de crer le rpertoire requis : un fichier portant le mme nom existe.");
							} catch (IOException e) {
								// TODO Auto-generated catch block
								e.printStackTrace();
							}
					}
					else if (command[0].equals("refresh"))
						cd(".");
					else if (command[0].equals("lpwd"))
						System.out.println(System.getProperty("user.dir"));
					else if (command[0].equals("info"))
						infos(entd);
					else if (command[0].equals("exit"))
						break;
					else if (!command[0].isEmpty() || command.length != 1)
						System.err.println("ENTDownloader: "+ command[0] +": command not found");
				}
			} catch (UnknownHostException e3) {
				if (pg.isVisible())
					pg.setVisible(false);
				System.err.println("ENTDownloader: Service indisponible: Votre connexion Internet semble rencontrer un problme. Assurez-vous que votre ordinateur est connect  Internet.");
				return;
			}
			System.out.print(login + "@ent.u-clermont1.fr:"+entd.getDirectoryPath()+">");
		}
	}
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		new Main();
	}
	
	private static String[] splitCommand(String command) throws ParseException {
		List<String> argv = new LinkedList<String>();
		int argc = 0;
		command = command.trim();
		for (int i = 0; i <command.length(); ++i) {
			char c = command.charAt(i);
			if(c==' ') {
				if (i>0 && command.charAt(i-1) != ' ')
					++argc;
				else
					continue;
			}
			else if (c=='"' && (i==0 || command.charAt(i-1) != '\\')) {
				int nexti;
				for(nexti = command.indexOf('"', i+1); ;nexti = command.indexOf('"', nexti+1)) {
					if (nexti == -1) throw new ParseException("Unexpected end of command", 0);
					if (command.charAt(nexti -1) != '\\') break;
				}
				argv.add(argv.size(), command.substring(i+1, nexti));
				i = nexti;
			}
			else {
				if(argc >= argv.size())
					argv.add(argc, "" + c);
				else
					argv.set(argc, (argc >= argv.size())?"":argv.get(argc) + c);
			}
		}
		return argv.toArray(new String[0]);
	}

	public void ls(List<FS_Element> directoryContent) {
		int nbdir = 0, nbfile = 0;
		for (FS_Element e : directoryContent){
			if (e.isDirectory()){
				System.out.println("Dossier " + e.getName());
				++nbdir;
			}
			else {
				System.out.println("Fichier " + e.getName());
				++nbfile;
			}
		}
		System.out.println("\nTotal : " + nbfile + " Fichier(s), " + nbdir + " Dossier(s)");
	}
	
	public void cd(String name) throws UnknownHostException {
		try {
			entd.changeDirectory(name);
		} catch (ENTUnauthenticatedUserException e) {
			if(Config.autoLogin) {
				String currentPath = entd.getDirectoryPath();
				if(!entd.login(login, password)) { //TODO Test expiration de session et gnralisation aux autres mthodes.
					pg.setVisible(false);
					System.err.println("La session a expire et la tentative de reconnection a chou. Veuillez vous reconnectez manuellement en relanant le programme.");
					System.exit(1);
				} else {
					cd("/");
					cd((ENTPath.isAbsolute(name)?"":(currentPath + "/")) + name); //TODO Risque de boucle !! (Peu probable mais pas impossible)
					pg.setVisible(false);
				}
			}
			else {
				System.err.println("La session a expire, veuillez vous reconnectez en relanant le programme."); 
				System.exit(1);
			}
				
		} 
		catch (ENTDirectoryNotFoundException e) {
			if(pg.isVisible())
				pg.setVisible(false);
			System.err.println("ENTDownloader: "+ e.getMessage() +": Aucun rpertoire de ce type");
		} catch (ENTInvalidFS_ElementTypeException e) {
			if(pg.isVisible())
				pg.setVisible(false);
			System.err.println("ENTDownloader: "+ e.getMessage() +": N'est pas un rpertoire");
		} catch (ParseException e) {
			if(pg.isVisible())
				pg.setVisible(false);
			System.err.println("ENTDownloader: "+ name +": Une erreur est survenue lors de l'accs  ce rpertoire");
		}
	}
	
	public void get(String name, String destination) {
		if(name == null || name.isEmpty()){
			System.err.println("ENTDownloader: get: Nom de fichier manquant");
			return;
		}
		try {
			entd.getFile(name, destination);
		}
		catch (ENTFileNotFoundException e) {
				System.err.println("ENTDownloader: "+ name +": Aucun fichier de ce type");
		}
		catch (FileNotFoundException e1) {
			System.err.println("ENTDownloader: "+ destination +": Accs refus. Impossible d'crire au chemin indiqu. Veuillez vrifier qu'un fichier" +
					" portant ce nom n'existe pas dj et que vous avez la permission d'crire  cet emplacement.");
		}
		catch (ENTInvalidFS_ElementTypeException e2) {
			System.err.println("ENTDownloader: "+ name +": N'est pas un fichier");
		} catch (IOException e3) {
			e3.getLocalizedMessage();
		}
	}
	private void infos(ENTDownloader entd) {
		System.out.println(productName + "  _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _");
		System.out.println("Version : " + productVersion);
		System.out.println("Auteur  : " + productAuthor);
		System.out.println("Contact : " + productContact);
		
		System.out.println("\nEspace de stockage _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _");
		System.out.print("Utilisateur       : ");
		if (entd.getUsername() != null) {
			System.out.print(entd.getUsername());
			if (entd.getLogin() != null) 
				System.out.print(" (" + entd.getLogin() + ")");
			System.out.println();
		}
		else if (entd.getLogin() != null)
			System.out.println(entd.getLogin());
		else
			System.out.println("Inconnu");
		if (entd.getUsedSpace() != -1 && entd.getCapacity() != -1) {
			System.out.println("Espace utilis    : " + entd.getUsedSpace() + " Mo");
			System.out.println("Espace disponible : " + (entd.getCapacity() - entd.getUsedSpace()) + " Mo");
			System.out.println("Espace total      : " + entd.getCapacity() + " Mo");
		}
		System.out.println();
	}

}

class PwdConsole {
	 public static String getPasswd(String prompt) throws Exception {
	  ConsoleEraser consoleEraser = new ConsoleEraser();
	  System.out.print(prompt);
	  BufferedReader stdin = new BufferedReader(new InputStreamReader(System.in));
	  consoleEraser.start();
	  System.out.print("\b");                       
	  String pass = stdin.readLine();
	  consoleEraser.halt();
	  System.out.print("\b");
	  return pass;
	  }
	 }

	class ConsoleEraser extends Thread {
	 private boolean running = true;
	 public void run() {
	  while (running) {
	   System.out.print("\b ");
	   }
	 }

	 public synchronized void halt() {
	  running = false;
	 }
	}
